part of rad_textures;

double bias(double a, double b)
{
  return math.pow(a, math.log(b) / math.log(0.5));
}

double gain(double a, double b)
{
  double p = math.log(1.0 - b) / math.log(0.5);

  if (a < .001)
    return 0.0;
  else if (a > .999)
    return 1.0;
  if (a < 0.5)
    return math.pow(2 * a, p) / 2;
  else
    return 1.0 - math.pow(2 * (1.0 - a), p) / 2;
}

double Step(double a, double x)
{
  return (x >= a) ? 1.0 : 0.0;
}

double Clamp(double x, double a, double b)
{
  return (x < a ? a : (x > b ? b : x));
}

double Min(double a, double b)
{
  return (a < b ? a : b);
}

double Max(double a, double b)
{
  return (a < b ? b : a);
}

double Abs(double x)
{
  return (x < 0 ? -x : x);
}

double SmoothStep(double a, double b, double x)
{
  if (x < a)
    return 0.0;
  if (x >= b)
    return 1.0;

  x = (x - a) / (b - a);
  return (x * x * (3 - 2 * x));
}

double Mod(double a, double b)
{
  num n = FASTFLOOR(a / b);

  a -= n * b;
  if (a < 0)
    a += b;

  return a;
}

double intersectRaySphere
(
 vec3 rayPosition, vec3 rayDirection,
 vec3 sphereCenter, double sphereRadius
 )
{
  vec3 oc = sphereCenter - rayPosition;
  double l2oc = dot(oc, oc);
  rayDirection.normalize();

  if (l2oc < sphereRadius) // starts inside sphere
  {
    double tca = dot(oc, rayDirection);
    double l2hc = (sphereRadius - l2oc) + tca * tca;
    return tca + math.sqrt(l2hc);
  }
  else // start outside sphere?
  {
    double tca = dot(oc, rayDirection);
    if (tca < 0) // point away from sphere
      return double.INFINITY;
    double l2hc = (sphereRadius - l2oc) + (tca * tca);
    if (l2hc > 0)
      return (tca - math.sqrt(l2hc));
    else
      return double.INFINITY;
  }
}
